package test.aop;

import java.lang.annotation.Annotation;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.reflect.MethodSignature;

import test.annotations.MethodAnnotation;

public aspect MethodAnnotationAspect {

    pointcut securedInvocation() : call(* test.*.print*(..)) && 
                           if( hasAnnotation( thisJoinPoint ) ) &&
                           !within(MethodAnnotationAspect);
    
    public static boolean hasAnnotation( JoinPoint jointPoint ) {
        
        System.out.println( "start ---> " + jointPoint.getSignature() );

        // find method annotation
        try {    
            Signature signature = jointPoint.getSignature();
            if ( signature instanceof MethodSignature ) {
                MethodSignature methodSignature = ( MethodSignature )signature;
                
                System.out.println( "\tMethod annotations size -> " + 
                                    methodSignature.getMethod().getAnnotations().length );

                Annotation annotation = 
                    methodSignature.getMethod().getAnnotation( MethodAnnotation.class );
                
                return annotation != null;            
            }
    
            return false;
        }
        finally {
            System.out.println( "end ---> " + jointPoint.getSignature() );
        }
    }
    
    before() : securedInvocation() {
        // do something 
    }
    
}
